Ext.namespace("dSS", "dSS.config", "dSS.devprops");
        // configuration values are set up and saved from the advanced form,
        // the standard form gives hints to the advanced form configuration

dSS.devprops.PBConfig = {
    "PUSHBUTTON_AREA1"  : -1,
    "PUSHBUTTON_AREA2"  : -2,
    "PUSHBUTTON_AREA3"  : -3,
    "PUSHBUTTON_AREA4"  : -4,
    "PUSHBUTTON_DEVICE" : -5,
    "PUSHBUTTON_ZONE"   : -6,
    "PUSHBUTTON_AREA"   : -7
};

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.GroupPushbuttonForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        this.areaSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 105,
            x: 240,
            y: 25,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA1, dSS._("Area 1")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA2, dSS._("Area 2")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA3, dSS._("Area 3")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA4, dSS._("Area 4")]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });


        this.pbradio = new Ext.form.RadioGroup({
                    //xtype: 'radiogroup',
                    //ref: 'radio',
                    fieldLabel: dSS._("Pushbutton"),
                    width: 245,
                    columns: 1,
                    items:
                    [
                        {
                            disabled: !config.deviceConfig,
                            boxLabel: dSS._("device pushbutton"),
                            name: 'rb_pushbutton_' + config.dsid,
                            id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_DEVICE,
                            inputValue : dSS.devprops.PBConfig.PUSHBUTTON_DEVICE
                        },
                        {
                            boxLabel: dSS._("area pushbutton"),
                            name: 'rb_pushbutton_' + config.dsid,
                            id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_AREA,
                            inputValue : dSS.devprops.PBConfig.PUSHBUTTON_AREA
                        },
                        {
                            boxLabel: dSS._("zone pushbutton"),
                            name: 'rb_pushbutton_' + config.dsid,
                            id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                            inputValue : dSS.devprops.PBConfig.PUSHBUTTON_ZONE
                        }
                    ]
                });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 80,
                    width: 355,
                    layout: 'absolute',
                    items:
                    [
                        {
                            xtype: 'panel',
                            border: false,
                            width: 355,
                            x: 0,
                            y: 0,
                            layout: 'form',
                            items: [ this.pbradio ]
                        },
                        this.areaSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.GroupPushbuttonForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.pbradio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var val = data.json.result[this.dsid][0].button[0].id;
                try {
                    var dec = this.decodeConfig(val);
                    this.pbradio.setValue('rb_pb_' + dec[0], dec[0]);
                    if (dec[0] == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        this.areaSelection.setValue(dec[1]);
                    }
                } catch (ex) {
                    if (this.closed === true) {
                        return;
                    }

                    this.pbradio.markInvalid(ex);
                }
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.areaSelection.on('select', function(combo, record, index) {
                var v = combo.getValue();
                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.pbradio.on('change', function(group, button) {
                var v = button.getRawValue();
                if (v == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                    if (this.areaSelection.getRawValue() == "") {
                        this.areaSelection.setValue(dSS.devprops.PBConfig.PUSHBUTTON_AREA1);
                    }

                    v = this.areaSelection.getValue();
                    this.areaSelection.enable();
                } else {
                    this.areaSelection.disable();
                }
                if (this.firstLoad === true) {
                    this.firstLoad = false;
                    return;
                }

                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.on('submit', function() {
            // data will be saved by advanced form
            this.fireEvent('formsubmitted');
        },
        this);
    },

    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var rbval;
        var dec = (value & 0x0000000f);
        var ext;
        switch (dec)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");
        }

        return [ rbval, ext ];
    }
});

Ext.reg('dssdevgrouppushbuttonform', dSS.devprops.GroupPushbuttonForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.GroupPushbuttonAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded');

        this.zonePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS._("Scenes 0-4"),
                dSS._("Scenes 2-4"),
                dSS._("Scene 1")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS._("Scenes 10-14"),
                dSS._("Scenes 12-14"),
                dSS._("Scene 11")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS._("Scenes 20-24"),
                dSS._("Scenes 22-24"),
                dSS._("Scene 21")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS._("Scenes 30-34"),
                dSS._("Scenes 32-34"),
                dSS._("Scene 31")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS._("Scenes 40-44"),
                dSS._("Scenes 42-44"),
                dSS._("Scene 41")
            ]
        ];

        this.areaPB =
        [
            // area 1
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Area 1")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                    dSS._("Scenes 12-14"),
                    dSS._("Scenes 12-14"),
                    dSS._("Area 1")
                ]
            ],
            // area 2
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 2")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                    dSS._("Scenes 22-24"),
                    dSS._("Scenes 22-24"),
                    dSS._("Area 2")
                ]
            ],
            // area 3
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 3")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                    dSS._("Scenes 32-34"),
                    dSS._("Scenes 32-34"),
                    dSS._("Area 3")
                ]
            ],
            // area 4
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 4")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP8,
                    dSS._("Scenes 42-44")
                ]
            ]
        ];

        this.devicePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS._("Scenes 2-4"),
                dSS._("Scenes 2-4"),
                dSS._("Device")

            ]
        ];

        this.sceneSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            autoSelect: false,
            editable: false,
            width: 170,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText',
                    'label2Text',
                    'label1Text'
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            fieldLabel: dSS._("Used scenes")
        });

        this.label1 = new Ext.form.Label({
             xtype: 'label',
             fieldLabel: dSS._("1x Click")
         });

        this.label2 = new Ext.form.Label({
            xtype: 'label',
            fieldLabel: dSS._("2x, 3x, 4x Click")
        });

        this.localPrio = new Ext.form.Checkbox({
            boxLabel: dSS._("Local priority for 1x click")
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            border: false,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 110,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Pushbutton") + ':'
                        },
                        this.sceneSelection,
                        this.label1,
                        this.label2,
                        this.localPrio
                    ]
                }
            ]
        });

        dSS.devprops.GroupPushbuttonAdvancedForm.superclass.constructor.apply(this, arguments);


        dSS.MessageBus.addListener('pushbuttonstdformchanged',
                          this.updateForm, this);

        this.sceneSelection.on('select', function(combo, record, index) {
                this.label1.setText(record.get('label1Text'));
                this.label2.setText(record.get('label2Text'));
            },this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                this.loadedValue = undefined;

                if (data.json.ok !== true) {
                    this.sceneSelection.markInvalid(data.json.message);
                    this.label1.setText("");
                    this.label2.setText("");
                    this.localPrio.setDisabled(true);
                    this.fireEvent('formloaded', true);
                    return;
                }

                try {
                    var val = data.json.result[this.dsid][0].button[0].id;
                    var prio = data.json.result[this.dsid][0].button[0].setsLocalPriority;

                    var load = this.decodeConfig(val);
                    this.sceneSelection.getStore().loadData(load[0]);
                    this.sceneSelection.setValue(load[1]);
                    this.label1.setText(load[2]);
                    this.label2.setText(
                        this.sceneSelection.getStore().getById(load[1]).get(
                                                                'displayText'));
                    this.loadedValue = load[1];
                    this.loadedValueLP = prio;
                    this.localPrio.setValue(prio);
                    this.localPrio.setDisabled(!load[3]);
                } catch (ex) {
                    if (this.closed === true) {
                        return;
                    }

                    this.sceneSelection.markInvalid(ex);
                    this.label1.setText("");
                    this.label2.setText("");
                }
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                dSS.MessageBus.removeListener('pushbuttonstdformchanged',
                                              this.updateForm, this);
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value = this.sceneSelection.getValue();
            var prio = this.localPrio.getValue();

            if ((value === this.loadedValue) &&
                (prio === this.loadedValueLP)) {
                this.fireEvent('formsubmitted');
                return;
            }

            var id = this.dsid;

            if (value !== this.loadedValue) {
                // we know this form is never used alone, it's always in the
                // same dialog as the output mode form; in order to not spam
                // the bus with two simultanious requests we will delay saving
                // of pushbutton settings by one second
                dSS.util.genericRequest.defer(1000,
                    this,
                    [
                        {
                            url: '/json/device/setButtonID',
                            params:
                            {
                                dsid : this.dsid,
                                buttonID : value
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var s = dSS.data.Loader.getDeviceStore();
                                var rec = s.getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonID', value);
                                    rec.commit();
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        }
                    ]
                );
            }


            if (prio != this.loadedValueLP) {
                dSS.util.genericRequest.defer(1500,
                    this,
                    [
                        {
                            url: '/json/property/setBoolean',
                            params:
                            {
                                path: '/apartment/zones/zone0/devices/' +
                                      this.dsid +
                                      '/button/setsLocalPriority',
                                value: prio
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        }
                    ]
                );
            }
        },
        this);
    },
    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var ret;
        var l1;
        var dec = (value & 0x0000000f);
        var showLocalPriority = false;
        switch (dec)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                ret = this.devicePB;
                l1 = dSS._("Device");
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                ret = this.areaPB[0];
                l1 = dSS._("Area 1");
                showLocalPriority = true;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                ret = this.areaPB[1];
                l1 = dSS._("Area 2");
                showLocalPriority = true;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                ret = this.areaPB[2];
                l1 = dSS._("Area 3");
                showLocalPriority = true;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                ret = this.areaPB[3];
                l1 = dSS._("Area 4");
                showLocalPriority = true;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                l1 = this.zonePB[0][3];
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                if (l1 === undefined) {
                    l1 = this.zonePB[1][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                if (l1 === undefined) {
                    l1 = this.zonePB[2][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                if (l1 === undefined) {
                    l1 = this.zonePB[3][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                if (l1 === undefined) {
                    l1 = this.zonePB[4][3];
                }
                ret = this.zonePB;
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");

        }
        return [ ret, dec, l1, showLocalPriority ];
    },
    updateForm: function(value) {
        if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA1) {
                this.sceneSelection.getStore().loadData(this.areaPB[0]);
                this.sceneSelection.setValue(this.areaPB[0][0][0]);
                this.label1.setText(dSS._("Area 1"));
                this.label2.setText(this.areaPB[0][0][1]);
                this.localPrio.setDisabled(false);
                this.localPrio.setValue(1);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA2) {
                this.sceneSelection.getStore().loadData(this.areaPB[1]);
                this.sceneSelection.setValue(this.areaPB[1][0][0]);
                this.label1.setText(dSS._("Area 2"));
                this.label2.setText(this.areaPB[1][0][1]);
                this.localPrio.setDisabled(false);
                this.localPrio.setValue(1);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA3) {
                this.sceneSelection.getStore().loadData(this.areaPB[2]);
                this.sceneSelection.setValue(this.areaPB[2][0][0]);
                this.label1.setText(dSS._("Area 3"));
                this.label2.setText(this.areaPB[2][0][1]);
                this.localPrio.setDisabled(false);
                this.localPrio.setValue(1);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA4) {
                this.sceneSelection.getStore().loadData(this.areaPB[3]);
                this.sceneSelection.setValue(this.areaPB[3][0][0]);
                this.label1.setText(dSS._("Area 4"));
                this.label2.setText(this.areaPB[3][0][1]);
                this.localPrio.setDisabled(false);
                this.localPrio.setValue(1);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
                this.sceneSelection.getStore().loadData(this.devicePB);
                this.sceneSelection.setValue(this.devicePB[0][0]);
                this.label1.setText(dSS._("Device"));
                this.label2.setText(this.devicePB[0][1]);
                this.localPrio.setDisabled(true);
                this.localPrio.setValue(1);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
                this.sceneSelection.getStore().loadData(this.zonePB);
                this.sceneSelection.setValue(this.zonePB[0][0]);
                this.label1.setText(this.zonePB[0][3]);
                this.label2.setText(this.zonePB[0][2]);
                this.localPrio.setDisabled(true);
                this.localPrio.setValue(0);
        } else {
                this.sceneSelection.getStore().loadData([]);
                this.sceneSelection.setValue();
                this.sceneSelection.markInvalid(
                        dSS._("Unknown pushbutton configuration!"));
                this.label1.setText("");
                this.label2.setText("");
                this.localPrio.setDisabled(true);
        }
    }
});

Ext.reg('dssdevgrouppushbuttonadvancedform', dSS.devprops.GroupPushbuttonAdvancedForm);

